@@ -1,7 +1,13 @@
 import array
+import logging
+import posixpath
+import warnings
 from collections.abc import MutableMapping
+from functools import cached_property
 
-from .core import url_to_fs
+from fsspec.core import url_to_fs
+
+logger = logging.getLogger("fsspec.mapping")
 
 
 class FSMap(MutableMapping):
@@ -34,9 +40,8 @@ class FSMap(MutableMapping):
 
     def __init__(self, root, fs, check=False, create=False, missing_exceptions=None):
         self.fs = fs
-        self.root = fs._strip_protocol(root).rstrip(
-            "/"
-        )  # we join on '/' in _key_to_str
+        self.root = fs._strip_protocol(root).rstrip("/")
+        self._root_key_to_str = fs._strip_protocol(posixpath.join(root, "x"))[:-1]
         if missing_exceptions is None:
             missing_exceptions = (
                 FileNotFoundError,
@@ -52,14 +57,22 @@ class FSMap(MutableMapping):
         if check:
             if not self.fs.exists(root):
                 raise ValueError(
-                    "Path %s does not exist. Create "
-                    " with the ``create=True`` keyword" % root
+                    f"Path {root} does not exist. Create "
+                    f" with the ``create=True`` keyword"
                 )
             self.fs.touch(root + "/a")
             self.fs.rm(root + "/a")
 
+    @cached_property
+    def dirfs(self):
+        """dirfs instance that can be used with the same keys as the mapper"""
+        from .implementations.dirfs import DirFileSystem
+
+        return DirFileSystem(path=self._root_key_to_str, fs=self.fs)
+
     def clear(self):
         """Remove all keys below root - empties out mapping"""
+        logger.info("Clear mapping at %s", self.root)
         try:
             self.fs.rm(self.root, True)
             self.fs.mkdir(self.root)
@@ -120,11 +133,16 @@ class FSMap(MutableMapping):
 
     def _key_to_str(self, key):
         """Generate full path for the key"""
-        if isinstance(key, (tuple, list)):
-            key = str(tuple(key))
-        else:
+        if not isinstance(key, str):
+            # raise TypeError("key must be of type `str`, got `{type(key).__name__}`"
+            warnings.warn(
+                "from fsspec 2023.5 onward FSMap non-str keys will raise TypeError",
+                DeprecationWarning,
+            )
+            if isinstance(key, list):
+                key = tuple(key)
             key = str(key)
-        return self.fs._strip_protocol("/".join([self.root, key]) if self.root else key)
+        return f"{self._root_key_to_str}{key}"
 
     def _str_to_key(self, s):
         """Strip path of to leave key name"""
@@ -142,6 +160,7 @@ class FSMap(MutableMapping):
         return result
 
     def pop(self, key, default=None):
+        """Pop data"""
         result = self.__getitem__(key, default)
         try:
             del self[key]
@@ -171,7 +190,7 @@ class FSMap(MutableMapping):
     def __contains__(self, key):
         """Does key exist in mapping?"""
         path = self._key_to_str(key)
-        return self.fs.exists(path) and self.fs.isfile(path)
+        return self.fs.isfile(path)
 
     def __reduce__(self):
         return FSMap, (self.root, self.fs, False, False, self.missing_exceptions)
@@ -184,7 +203,7 @@ def maybe_convert(value):
             # The buffer interface doesn't support datetime64/timdelta64 numpy
             # arrays
             value = value.view("int64")
-        value = bytearray(memoryview(value))
+        value = bytes(memoryview(value))
     return value
 
 
